/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package com.supinfo.spitfireboard.jpa;

import com.supinfo.spitfireboard.dao.PostDao;
import com.supinfo.spitfireboard.entities.Post;
import java.util.Date;
import java.util.List;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

/**
 *
 * @author Fabien LOISON
 */
@Stateless
public class JpaPostDao implements PostDao {
    @PersistenceContext
    private EntityManager em;
    
    @Override
    public Post getById(Long id) {
        CriteriaBuilder builder = em.getCriteriaBuilder();
        CriteriaQuery<Post> query = builder.createQuery(Post.class);
        Root<Post> user = query.from(Post.class);
        query.where(builder.equal(user.get("id"), id));
        return em.createQuery(query).getSingleResult();
    }
    
    @Override
    public List<Post> getPostsByTopicId(Long topicId) {
        CriteriaBuilder builder = em.getCriteriaBuilder();
        CriteriaQuery<Post> query = builder.createQuery(Post.class);
        Root<Post> post = query.from(Post.class);
        query.where(builder.equal(post.get("topicId"), topicId));
        query.orderBy(builder.asc(post.get("creationDate")));
        return em.createQuery(query).getResultList();
    }
    
    @Override
    public List<Post> getPostsByUserId(Long userId) {
        CriteriaBuilder builder = em.getCriteriaBuilder();
        CriteriaQuery<Post> query = builder.createQuery(Post.class);
        Root<Post> post = query.from(Post.class);
        query.where(builder.equal(post.get("userId"), userId));
        query.orderBy(builder.desc(post.get("creationDate")));
        return em.createQuery(query).setMaxResults(10).getResultList();
    }
    
    @Override 
    public List<Post> getAll() {
        CriteriaBuilder builder = em.getCriteriaBuilder();
        CriteriaQuery<Post> query = builder.createQuery(Post.class);
        query.from(Post.class);
        return em.createQuery(query).getResultList();
    }
    
    @Override
    public Post getFirstPost() {
        CriteriaBuilder builder = em.getCriteriaBuilder();
        CriteriaQuery<Post> query = builder.createQuery(Post.class);
        Root<Post> post = query.from(Post.class);
        query.from(Post.class);
        query.orderBy(builder.asc(post.get("creationDate")));
        List<Post> result = em.createQuery(query).setMaxResults(1).getResultList();
        return result.get(0);
    }
    
    @Override
    public Long persist(Post post) {
        em.persist(post);
        em.flush();
        return post.getId();
    }
    
    @Override
    public void delete(Long id) {
        Post post = this.getById(id);
        em.remove(post);
    }
}
